@@ -152,13 +152,13 @@ do_password_callback (const char *prompt)
   debugprintf ("-> do_password_callback\n");
   Connection_end_allow_threads (tls->g_current_connection);
   args = Py_BuildValue ("(s)", prompt);
-  result = PyEval_CallObject (cups_password_callback, args);
+  result = PyEval_CallObject (tls->cups_password_callback, args);
   Py_DECREF (args);
   if (result == NULL)
   {
-    debugprintf ("<- do_password_callback (empty string)\n");
+    debugprintf ("<- do_password_callback (exception)\n");
     Connection_begin_allow_threads (tls->g_current_connection);
-    return "";
+    return NULL;
   }
 
   if (password) {
@@ -166,14 +166,20 @@ do_password_callback (const char *prompt)
     password = NULL;
   }
 
-  pwval = PyString_AsString (result);
-  password = strdup (pwval);
-  Py_DECREF (result);
-  if (!password)
+  if (result == Py_None)
+    password = NULL;
+  else
   {
-    debugprintf ("<- do_password_callback (empty string)\n");
+    pwval = PyString_AsString (result);
+    password = strdup (pwval);
+  }
+
+  Py_DECREF (result);
+  if (!password || !*password)
+  {
+    debugprintf ("<- do_password_callback (empty/null)\n");
     Connection_begin_allow_threads (tls->g_current_connection);
-    return "";
+    return NULL;
   }
 
   Connection_begin_allow_threads (tls->g_current_connection);
@@ -220,8 +226,6 @@ cups_modelSort (PyObject *self, PyObject *args)
     return NULL;
   }
 
-  wca = malloc (size_a);
-
   len_b = 1 + PyUnicode_GetSize (b);
   size_b = len_b * sizeof (wchar_t);
   if ((size_b / sizeof (wchar_t)) != len_b) {
@@ -231,8 +235,8 @@ cups_modelSort (PyObject *self, PyObject *args)
     return NULL;
   }
 
+  wca = malloc (size_a);
   wcb = malloc (size_b);
-  
   if (wca == NULL || wcb == NULL) {
     Py_DECREF (a);
     Py_DECREF (b);
@@ -339,8 +343,10 @@ cups_setPasswordCB (PyObject *self, PyObject *args)
   }
 
   debugprintf ("-> cups_setPasswordCB\n");
+#ifdef HAVE_CUPS_1_4
   Py_XDECREF (tls->cups_password_callback_context);
   tls->cups_password_callback_context = NULL;
+#endif /* HAVE_CUPS_1_4 */
 
   Py_XINCREF (cb);
   Py_XDECREF (tls->cups_password_callback);
@@ -513,8 +519,8 @@ static PyMethodDef CupsMethods[] = {
     "setPasswordCB(fn) -> None\n\n"
     "Set password callback function.  This Python function will be called \n"
     "when a password is required.  It must take one string parameter \n"
-    "(the password prompt) and it must return a string (the password).  To \n"
-    "abort the operation it may return the empty string ('').\n\n"
+    "(the password prompt) and it must return a string (the password), or \n"
+    "None to abort the operation.\n\n"
     "@type fn: callable object\n"
     "@param fn: callback function" },
 
@@ -523,10 +529,10 @@ static PyMethodDef CupsMethods[] = {
     "setPasswordCB2(fn, context=None) -> None\n\n"
     "Set password callback function.  This Python function will be called \n"
     "when a password is required.  It must take parameters of type string \n"
-    "(the password prompt), instance (the cups.Connection), string (the HTTP "
-    "method), string (the HTTP resource) and, optionally, the user-supplied "
-    "context.  It must return a string (the password).  To \n"
-    "abort the operation it may return the empty string ('').\n\n"
+    "(the password prompt), instance (the cups.Connection), string (the \n"
+    "HTTP method), string (the HTTP resource) and, optionally, the user-\n"
+    "supplied context.  It must return a string (the password), or None \n"
+    "to abort the operation.\n\n"
     "@type fn: callable object, or None for default handler\n"
     "@param fn: callback function" },
 #endif /* HAVE_CUPS_1_4 */
@@ -681,6 +687,15 @@ initcups (void)
   INT_CONSTANT (HTTP_REQUEST_TIMEOUT);
   INT_CONSTANT (HTTP_UPGRADE_REQUIRED);
   INT_CONSTANT (HTTP_SERVER_ERROR);
+  INT_CONSTANT (HTTP_NOT_IMPLEMENTED);
+  INT_CONSTANT (HTTP_BAD_GATEWAY);
+  INT_CONSTANT (HTTP_SERVICE_UNAVAILABLE);
+  INT_CONSTANT (HTTP_GATEWAY_TIMEOUT);
+  INT_CONSTANT (HTTP_NOT_SUPPORTED);
+  INT_CONSTANT (HTTP_AUTHORIZATION_CANCELED);
+#if CUPS_VERSION_MAJOR > 1 || (CUPS_VERSION_MAJOR == 1 && CUPS_VERSION_MINOR >= 5)
+  INT_CONSTANT (HTTP_PKI_ERROR);
+#endif /* CUPS 1.5 */
 
   // PPD UI enum
   INT_CONSTANT (PPD_UI_BOOLEAN);
@@ -800,6 +815,10 @@ initcups (void)
   INT_CONSTANT (IPP_ERROR_JOB_CANCELLED);
   INT_CONSTANT (IPP_MULTIPLE_JOBS_NOT_SUPPORTED);
   INT_CONSTANT (IPP_PRINTER_IS_DEACTIVATED);
+#if CUPS_VERSION_MAJOR > 1 || (CUPS_VERSION_MAJOR == 1 && CUPS_VERSION_MINOR >= 5)
+  INT_CONSTANT (IPP_AUTHENTICATION_CANCELED);
+  INT_CONSTANT (IPP_PKI_ERROR);
+#endif /* CUPS 1.5 */
 
   // IPP states
   INT_CONSTANT (IPP_ERROR);
